@@ -3,15 +3,17 @@ package App::RPi::EnvUI;
 use App::RPi::EnvUI::API;
 use Data::Dumper;
 use Dancer2;
-use Dancer2::Plugin::Database;
+use Dancer2::Plugin::Auth::Extensible;
+#use Dancer2::Session::JSON;
 use Mock::Sub no_warnings => 1;
 
-our $VERSION = '0.24';
+our $VERSION = '0.25';
 
 my $db = App::RPi::EnvUI::DB->new;
 my $api = App::RPi::EnvUI::API->new(db => $db);
 
 my $log = $api->log()->child('webapp');
+
 $api->_reset();
 $api->_config_light();
 $api->env($api->read_sensor);
@@ -26,6 +28,17 @@ get '/' => sub {
         # return template 'main';
         return template 'test';
     };
+
+post '/login' => sub {
+        session user => params->{username};
+        session pass => $api->user(params->{username})->{pass};
+        redirect '/';
+    };
+
+any '/logout' => sub {
+        app->destroy_session;
+    };
+
 get '/light' => sub {
         my $log = $log->child('/light');
         $log->_7("entered");
@@ -109,6 +122,7 @@ get '/set_aux/:aux/:state' => sub {
         };
     };
 
+
 true;
 
 __END__
