@@ -3,6 +3,7 @@ package App::RPi::EnvUI::API;
 use App::RPi::EnvUI::DB;
 use App::RPi::EnvUI::Event;
 use Carp qw(confess);
+use Crypt::SaltedHash;
 use Data::Dumper;
 use DateTime;
 use JSON::XS;
@@ -10,35 +11,35 @@ use Logging::Simple;
 use Mock::Sub no_warnings => 1;
 use RPi::WiringPi::Constant qw(:all);
 
-our $VERSION = '0.24';
+our $VERSION = '0.25';
 
 # mocked sub handles for when we're in testing mode
 
-my ($temp_sub, $hum_sub, $wp_sub, $pm_sub);
+our ($temp_sub, $hum_sub, $wp_sub, $pm_sub);
 
 # class variables
 
 my $master_log;
 my $log;
 my $sensor;
+my $events;
 
-# public
+# public environment methods
 
 sub new {
     my $self = bless {}, shift;
-    $self->_args(@_);
+
+    my $caller = (caller)[0];
+    $self->_args(@_, caller => $caller);
 
     $self->_log;
-    $self->config;
     $self->_init;
 
-    $log->_6($self->config. " is the config file");
+    $log->_7("successfully initialized the system");
 
-    $self->_parse_config($self->config);
+    $self->events if ! $self->testing && ! defined $events;
 
-    $log->_7("successfully parsed the config file");
-
-    $self->events if ! $self->testing;
+    $log->_7("successfully started the async events");
 
     return $self;
 }
@@ -278,19 +279,6 @@ sub aux_time {
     my $on_length = time() - $on_time;
     return $on_time == 0 ? 0 : $on_length;
 }
-sub config {
-    $_[0]->{config_file} = $_[1] if defined $_[1];
-    return $_[0]->{config_file} || 'config/envui.json';
-}
-sub db {
-    my ($self, $db) = @_;
-    $self->{db} = $db if defined $db;
-    return $self->{db};
-}
-sub debug_sensor {
-     $_[0]->{debug_sensor} = $_[1] if defined $_[1];
-    return $_[0]->{debug_sensor};
-}
 sub env {
     my ($self, $temp, $hum) = @_;
 
@@ -312,49 +300,13 @@ sub env {
     }
 
     my $ret = $self->db()->env;
-    return {temp => 0, humidity => 0} if ! defined $ret;
+    return {temp => -1, humidity => -1} if ! defined $ret;
     return $self->db()->env;
 }
-sub env_humidity_aux {
-    my $self = shift;
-    return $self->_config_control('humidity_aux');
-}
-sub env_temp_aux {
-    my $self = shift;
-    return $self->_config_control('temp_aux');
-}
-sub events {
-    my $self = shift;
-
-    my $log = $self->log('events');
-
-    my $events = App::RPi::EnvUI::Event->new($self->testing);
-
-    $self->{events}{env_to_db} = $events->env_to_db;
-    $self->{events}{env_action} = $events->env_action;
-
-    $self->{events}{env_to_db}->start;
-    $self->{events}{env_action}->start;
-
-    $log->_7("events successfully started");
-}
 sub humidity {
     my $self = shift;
     return $self->env()->{humidity};
 }
-sub log {
-    return $master_log;
-}
-sub log_file {
-     $_[0]->{log_file} = $_[1] if defined $_[1];
-    return $_[0]->{log_file};
-}
-sub log_level {
-     $_[0]->{log_level} = $_[1] if defined $_[1];
-    return defined $_[0]->{log_level}
-        ? $_[0]->{log_level}
-        : -1;
-}
 sub read_sensor {
     my $self = shift;
 
@@ -370,11 +322,6 @@ sub read_sensor {
 
     return ($temp, $hum);
 }
-sub sensor {
-    my ($self, $sensor) = @_;
-    $self->{sensor} = $sensor if defined $sensor;
-    return $self->{sensor};
-}
 sub switch {
     my ($self, $aux_id) = @_;
 
@@ -400,9 +347,147 @@ sub temp {
     my $self = shift;
     return $self->env()->{temp};
 }
+
+# public core operational methods
+
+sub auth {
+    my ($self, $user, $pw) = @_;
+
+    if (! defined $user){
+        confess "\n\nauth() requires a username sent in\n\n";
+    }
+
+    if (! defined $pw){
+        confess "\n\nauth() requires a password sent in\n\n";
+
+    }
+    my $csh = Crypt::SaltedHash->new(algorithm => 'SHA1');
+
+    my $crypted = $self->db()->user($user)->{pass};
+
+    return $csh->validate($crypted, $pw);
+}
+sub events {
+    my $self = shift;
+
+    my $log = $self->log('events');
+
+    $events = App::RPi::EnvUI::Event->new($self->testing);
+
+    $self->{events}{env_to_db} = $events->env_to_db;
+    $self->{events}{env_action} = $events->env_action;
+
+    $self->{events}{env_to_db}->start;
+    $self->{events}{env_action}->start;
+
+    $log->_7("events successfully started");
+}
+sub log {
+    my $self = shift;
+    $master_log->file($self->log_file) if $self->log_file;
+    $master_log->level($self->log_level);
+    return $master_log;
+}
+sub passwd {
+    my ($self, $pw) = @_;
+
+    if (! defined $pw){
+        confess "\n\nplain text password string required\n\n";
+    }
+
+    my $csh = Crypt::SaltedHash->new(
+        algorithm => 'SHA1',
+    );
+
+    $csh->add($pw);
+
+    my $salted = $csh->generate;
+
+    return $salted;
+}
+sub user {
+    my ($self, $un) = @_;
+
+    if (! defined $un){
+        confess "\n\nuser() requires a username to be sent in\n\n";
+    }
+
+    return $self->db()->user($un);
+}
+# public configuration getters
+
+sub env_humidity_aux {
+    my $self = shift;
+    return $self->_config_control('humidity_aux');
+}
+sub env_temp_aux {
+    my $self = shift;
+    return $self->_config_control('temp_aux');
+}
+
+# public instance variable methods
+
+sub config {
+    $_[0]->{config_file} = $_[1] if defined $_[1];
+    return $_[0]->{config_file} || 'config/envui.json';
+}
+sub db {
+    my ($self, $db) = @_;
+    $self->{db} = $db if defined $db;
+    return $self->{db};
+}
+sub debug_sensor {
+    my ($self, $bool) = @_;
+
+    if (defined $bool){
+        $self->{debug_sensor} = $bool;
+    }
+
+    return $self->{debug_sensor};
+}
+sub log_file {
+    my ($self, $fn) = @_;
+
+    if (defined $fn){
+        $self->{log_file} = $fn;
+    }
+
+    return $self->{log_file};
+}
+sub log_level {
+    my ($self, $level) = @_;
+
+    if (defined $level){
+        if ($level < -1 || $level > 7){
+            warn "log level has to be between 0 and 7... disabling logging\n";
+            $level = -1;
+        }
+        $self->{log_level} = $level;
+    }
+
+    return $self->{log_level};
+}
+sub sensor {
+    my ($self, $sensor) = @_;
+    $self->{sensor} = $sensor if defined $sensor;
+    return $self->{sensor};
+}
 sub testing {
-    $_[0]->{testing} = $_[1] if defined $_[1];
-    return $_[0]->{testing};
+    my ($self, $bool) = @_;
+
+    if (defined $bool){
+        $self->{testing} = $bool;
+    }
+    return $self->{testing};
+}
+sub test_mock {
+    my ($self, $mock) = @_;
+
+    if (defined $mock){
+        $self->{test_mock} = $mock;
+    }
+    $self->{test_mock} = 1 if ! defined $self->{test_mock};
+    return $self->{test_mock};
 }
 
 # private
@@ -414,6 +499,7 @@ sub _args {
     $self->log_file($args{log_file});
     $self->log_level($args{log_level});
     $self->testing($args{testing});
+    $self->test_mock($args{test_mock});
 }
 sub _bool {
     # translates javascript true/false to 1/0
@@ -431,9 +517,14 @@ sub _config_control {
 sub _config_core {
     my $self = shift;
     my $want = shift;
+
     if (! defined $self->db()){
         confess "API's DB object is not defined.";
     }
+
+    if (! defined $want){
+        confess "_config_core() requires a \$want param\n";
+    }
     return $self->db()->config_core($want);
 }
 sub _config_light {
@@ -488,82 +579,101 @@ sub _init {
 
     my $log = $log->child('_init()');
 
+    $self->db(
+        App::RPi::EnvUI::DB->new(
+            testing => $self->testing
+        )
+    );
+
     if ($self->_ui_test_mode || $self->testing){
-        $log->_6("testing mode");
-
-        $self->config('t/envui.json');
-
-        if ($self->_ui_test_mode) {
-            $log->_6("UI testing mode");
-
-            $self->testing(1);
-
-            my $mock = Mock::Sub->new;
-
-            $temp_sub = $mock->mock(
-                'RPi::DHT11::temp',
-                return_value => 80
-            );
-
-            $log->_7("mocked RPi::DHT11::temp");
-
-            $hum_sub = $mock->mock(
-                'RPi::DHT11::humidity',
-                return_value => 20
-            );
-
-            $log->_7("mocked RPi::DHT11::humidity");
-
-            $pm_sub = $mock->mock(
-                'App::RPi::EnvUI::API::pin_mode',
-                return_value => 'ok'
-            );
-
-            $wp_sub = $mock->mock(
-                'App::RPi::EnvUI::API::write_pin',
-                return_value => 'ok'
-            );
-
-            $log->_7(
-                "mocked WiringPi::write_pin as App::RPi::EnvUI::API::write_pin"
-            );
-        }
-
-        warn "API in test mode\n";
-
-        $self->sensor(bless {}, 'RPi::DHT11');
-
-        $log->_7("blessed a fake sensor");
-
-        $self->db(
-            App::RPi::EnvUI::DB->new(
-                testing => $self->testing
-            )
-        );
-
-        $log->_7("created a DB object with testing enabled");
+        $self->_test_mode
     }
     else {
-        if (! exists $INC{'WiringPi/API.pm'}){
-            require WiringPi::API;
-            WiringPi::API->import(qw(:perl));
-        }
-        if (! exists $INC{'RPi/DHT11.pm'}){
-            require RPi::DHT11;
-            RPi::DHT11->import;
-        }
-        $log->_6("required/imported WiringPi::API and RPi::DHT11");
+        $self->_prod_mode;
+    }
+}
+sub _test_mode {
+    my ($self) = @_;
 
+    my $log = $log->child('_test_mode');
+    $log->_6("testing mode");
+
+    $self->testing(1);
+    $self->db(App::RPi::EnvUI::DB->new(testing => 1));
+    $self->config('t/envui.json');
+    $self->_parse_config;
+
+
+    if ($self->test_mock) {
+        my $mock = Mock::Sub->new;
+
+        $temp_sub = $mock->mock(
+            'RPi::DHT11::temp',
+            return_value => 80
+        );
+
+        $log->_7( "mocked RPi::DHT11::temp" );
+
+        $hum_sub = $mock->mock(
+            'RPi::DHT11::humidity',
+            return_value => 20
+        );
+
+        $log->_7( "mocked RPi::DHT11::humidity" );
+
+        $pm_sub = $mock->mock(
+            'App::RPi::EnvUI::API::pin_mode',
+            return_value => 'ok'
+        );
+
+        $wp_sub = $mock->mock(
+            'App::RPi::EnvUI::API::write_pin',
+            return_value => 'ok'
+        );
+    }
+
+    $log->_7(
+        "mocked WiringPi::write_pin as App::RPi::EnvUI::API::write_pin"
+    );
+
+    warn "API in test mode\n";
+
+    $self->sensor(bless {}, 'RPi::DHT11');
+
+    $log->_7("blessed a fake sensor");
+
+}
+sub _prod_mode {
+    my ($self) = @_;
+
+    my $log = $log->child('_prod_mode');
+
+    $self->_parse_config;
+
+    if (! exists $INC{'WiringPi/API.pm'} && ! $self->testing){
+        require WiringPi::API;
+        WiringPi::API->import(qw(:perl));
+    }
+    if (! exists $INC{'RPi/DHT11.pm'} && ! $self->testing){
+        require RPi::DHT11;
+        RPi::DHT11->import;
+    }
+    $log->_6("required/imported WiringPi::API and RPi::DHT11");
+
+    if (! defined $sensor){
         $sensor =  RPi::DHT11->new(
             $self->_config_core('sensor_pin'), $self->debug_sensor
         );
-        $self->sensor($sensor);
-        $log->_6("instantiated a new RPi::DHT11 sensor object");
     }
+
+    $self->sensor($sensor);
+    $log->_6("instantiated a new RPi::DHT11 sensor object");
 }
 sub _log {
     my ($self) = @_;
 
+    # configures the class-level log
+
     $master_log = Logging::Simple->new(
         name => 'EnvUI',
         print => 1,
@@ -578,6 +688,10 @@ sub _parse_config {
 
     $config = $self->config if ! defined $config;
 
+    if (! -e $config){
+        confess "\n\nconfig file '$config' not found...\n\n";
+    }
+
     my $json;
     {
         local $/;
@@ -603,6 +717,14 @@ sub _parse_config {
                 'id',
                 $directive
             );
+
+            # populate some internal variables from the 'core'
+            # config section
+
+            if ($conf_section eq 'core'){
+                next if $directive eq 'testing';
+                $self->{$directive} = $conf->{$conf_section}{$directive};
+            }
         }
     }
 }
@@ -665,6 +787,14 @@ Optional, Bool. Send in C<1> to enable testing, C<0> to disable it.
 
 Default: C<0>
 
+    test_mock
+
+This flag is only useful when C<testing> param is set to true, and should only
+be used when writing unit tests for the L<App::RPi::EnvUI::Event> class. Due to
+the way the system works, the API has to avoid mocking out items in test mode,
+and the mocks have to be set within the test file itself. Do not use this flag
+unless you are writing unit tests.
+
     log_level
 
 Optional, Integer. Send in a level of C<0-7> to enable logging.
@@ -725,6 +855,23 @@ Parameters:
 Mandatory, String. The string name representation of the temperature auxillary.
 By default, this will be C<aux1>.
 
+=head2 auth($user, $pw)
+
+Checks whether a user is supplying the correct password.
+
+Parameters:
+
+    $user
+
+Mandatory, String. The user name to validate the password for.
+
+    $pw
+
+Mandatory, String. The plain text password to verify.
+
+
+Return: True (C<1>) if successful, C<undef> otherwise.
+
 =head2 aux($aux_id)
 
 Retrieves from the database a hash reference that contains the details of a
@@ -954,6 +1101,18 @@ Return: Integer, the current level.
 
 Default: C<-1> (logging disabled)
 
+=head2 passwd($pw)
+
+Generates an SHA-1 hashed password.
+
+Parameters:
+
+    $pw
+
+Mandatory, String. A plain text string (the password).
+
+Return: SHA-1 hashed password string.
+
 =head2 read_sensor
 
 Retrieves and returns the current temperature and humidity within an array of
@@ -999,6 +1158,33 @@ Optional, Bool. C<0> for production mode, and C<1> for testing mode.
 
 Return: Bool, whether we're in testing mode or not.
 
+=head2 test_mock($bool)
+
+This is for use internally when testing the L<App::RPi::EnvUI::Event> module.
+Normally, the API mocks out items for testing, but in C<Event>'s case, the test
+file itself has to do the mocking.
+
+Parameters:
+
+    $bool
+
+Optional, Bool. C<0> to disable, C<1> to enable.
+
+Default: C<1>, enabled (the API will mock in test mode)
+
+=head2 user($user)
+
+Fetches a user's details.
+
+Parameters:
+
+    $user
+
+Mandatory, String. The username of the user to fetch details for.
+
+Return: href, the hash reference containing user details per the 'user' database
+table.
+
 =head1 AUTHOR
 
 Steve Bertrand, E<lt>steveb@cpan.org<gt>
