@@ -5,7 +5,7 @@ use DateTime;
 use DBI;
 use RPi::WiringPi::Constant qw(:all);
 
-our $VERSION = '0.24';
+our $VERSION = '0.25';
 
 sub new {
     my ($class, %args) = @_;
@@ -16,17 +16,34 @@ sub new {
         ? 't/envui.db'
         : 'db/envui.db';
 
-        warn "DB in test mode\n" if $self->{testing};
+    $db_file = $self->{db_file} if defined $self->{db_file};
 
-        $self->{db} = DBI->connect(
-            "dbi:SQLite:dbname=$db_file",
-            "",
-            "",
-            {RaiseError => 1}
-        ) or die $DBI::errstr;
+    warn "DB in test mode\n" if $self->{testing};
+
+    $self->{db} = DBI->connect(
+        "dbi:SQLite:dbname=$db_file",
+        "",
+        "",
+        {RaiseError => $self->{db_err}}
+    ) or die $DBI::errstr;
 
     return $self;
 }
+sub user {
+    my ($self, $user) = @_;
+
+    my $sth = $self->{db}->prepare(
+        "SELECT * FROM auth WHERE user=?;"
+    );
+
+    $sth->execute($user);
+
+    my $res = $sth->fetchrow_hashref();
+
+    return ! ref $res eq 'HASH'
+        ? undef
+        : $res;
+}
 sub aux {
     my ($self, $aux_id) = @_;
 
@@ -171,6 +188,18 @@ Optional, Bool. C<1> to enable testing mode, C<0> to disable.
 
 Default: C<0> (off)
 
+=head2 user($user)
+
+Fetches a user's information as found in the 'auth' database table.
+
+Parameters:
+
+    $user
+
+Mandatory, String. The name of the user to fetch the password for.
+
+Return: A hash reference containing the user's details.
+
 =head2 aux($aux_id)
 
 Fetches and returns a hash reference containing the details of an auxillary
