@@ -53,9 +53,12 @@ CREATE TABLE core (
 INSERT INTO core VALUES ('event_fetch_timer', 15);
 INSERT INTO core VALUES ('event_action_timer', 3);
 INSERT INTO core VALUES ('event_display_timer', 4);
+INSERT INTO core VALUES ('time_zone', 'America/Edmonton');
 INSERT INTO core VALUES ('sensor_pin', -1);
 INSERT INTO core VALUES ('testing', 0);
-INSERT INTO core VALUES ('time_zone', 'America/Edmonton');
+INSERT INTO core VALUES ('debug_sensor', 0);
+INSERT INTO core VALUES ('log_file', "");
+INSERT INTO core VALUES ('log_level', -1);
 
 DROP TABLE IF EXISTS water;
 
@@ -79,3 +82,13 @@ INSERT INTO light VALUES ('on_hours', '12');
 INSERT INTO light VALUES ('on_since', 0);
 INSERT INTO light VALUES ('toggle', 'disabled');
 INSERT INTO light VALUES ('enable', 0);
+
+DROP TABLE IF EXISTS auth;
+
+CREATE TABLE auth (
+    user VARCHAR(50),
+    pass VARCHAR(50)
+);
+
+INSERT INTO auth VALUES ('admin', '{SSHA1}B3WvkiZINyUB1JiP83xQ4evOe9EsrpIK');
+
